//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)

// The maximum length of the input and output array "in" and "out" is 30.

// Las entradas del sistema son
// in[0]= fr;       frecuencia de la red
// in[1]= Vref;     tension de referencia sin normalizar
// in[2]= fs;       frecuencia de conmutacin: La frecuencia de muestreo es fm=fs
// in[3]= Vcc;      tension de la barra CC
// in[4]= vo;       tension de salida en bornes del capacitor del filtro LC
// in[5]= iL;       corriente del inductor del filtro LC
// in[6]= t_rampa;  tiempo final de la rampa de variacin de la referencia

#include "dll.h"
#include <windows.h>
#include "ganancias_inversor.h"

// Aqui se implementa una estrategia de control diseada en matlab, que consta de 
// un compensador proporcional para el lazo interno de control de corriente y de
// un compensador PI para el lazo externo de control de tensin.
// La frecuencia de muestreo de ambos lazos es de 10 kHz.

void __declspec(dllexport) simuser (double t, double delt, double *in, double *out)

{
   	static double Vbase=311;
   	static double Ibase=50;

   	static double vo=0;   // tension de fase en bornes del capacitor de filtro
   	static double iL=0;   // Corriente en el inductor.
   	static double uk=0,  upk=0, uc=0;
   	static double umi_1k=0, umi_1km1=0, umi_1km2=0;
   	static double umi_3k=0, umi_3km1=0, umi_3km2=0;
   	static double umi_5k=0, umi_5km1=0, umi_5km2=0;
   	static double umi_7k=0, umi_7km1=0, umi_7km2=0;
   	static double umi_9k=0, umi_9km1=0, umi_9km2=0;
   	static double umi_11k=0, umi_11km1=0, umi_11km2=0;
   	static double ek=0, ekm1=0, ekm2=0;
   	static double ref, rd=0, ri=0;
	static double uk_lim=0, uk_sat=132.0;  // uk_sat=132.0 para valor nominal de corriente: 23A - Estaba = 180
	static double rampa=0;
	static double nc=5;	  // N de periodos de la rampa
	static double muestra=0;
			
	// Variables enteras
	static int contador = 0, Ncont, calculo_u, muestreo, direccion = 1;
	static int PWM1, PWM2;
	static int ComparReg1, ComparReg2;
	static int i=0, j=0;
	static int samp=0;
	
	static double Vcc;
	Vcc=in[3];
	static double fn;
	fn=Vbase/Vcc;            /* Factor de normalizacin de la accin de control
							 /* aplicada a la planta */
	
	static double fr;        /* Frecuencia fundamental de la tensin de salida */
	fr=in[0];
	static double Ts;        /* Periodo de muestreo */
	Ts = 1/in[2];
	static double Tr;        /* Periodo fundamental de la tensin de salida */
	Tr=1/fr;
	
// Se calcula el numero de conteos en un periodo: Valor mximo del contador
   Ncont=Ts/delt;
   muestra=0;

   if (calculo_u==1)
      {
      calculo_u=0;
	  
	  // poner una senoidal con la amplitud deseada para la tension de referencia
      ref = in[1]/Vbase;  // in[1]/Vbase Agregar este trmino para implementacin en p.u.
      
      /*
	  rampa = t;
      // Variacin inicial de Vref (generacin de rampa):
      if (t < nc*Tr)
           {rd = (1/(nc*Tr))*rampa*ref;} // Calcula incremento de referencia.
      else
      	  {rd = ref;}
      */
	  	  
      rd = ref;  // Incorporar esta lnea si se desea realizar partida suave de Vcc
 	  /*********************************************************************************/
      /*                         Se calcula el error de tension                        */
      /*********************************************************************************/
      ek = rd - vo;
      /*********************************************************************************/
      /*                   Calculo de la estrategia de control                         */
      /*********************************************************************************/
      // Lazo de control de tensin con controladores resonantes + compensador P
    
      upk = Kp*ek;
      umi_1k = a2_1*ek + a1_1*ekm1 + a0_1*ekm2 - b1_1*umi_1km1 - b0_1*umi_1km2; 
      umi_3k = a2_3*ek + a1_3*ekm1 + a0_3*ekm2 - b1_3*umi_3km1 - b0_3*umi_3km2;
      umi_5k = a2_5*ek + a1_5*ekm1 + a0_5*ekm2 - b1_5*umi_5km1 - b0_5*umi_5km2;
      umi_7k = a2_7*ek + a1_7*ekm1 + a0_7*ekm2 - b1_7*umi_7km1 - b0_7*umi_7km2;
      umi_9k = a2_9*ek + a1_9*ekm1 + a0_9*ekm2 - b1_9*umi_9km1 - b0_9*umi_9km2;
      umi_11k= a2_11*ek + a1_11*ekm1 + a0_11*ekm2 - b1_11*umi_11km1 - b0_11*umi_11km2;
      
      uk=upk + umi_1k + umi_3k + umi_5k + umi_7k + umi_9k + umi_11k; 
	  
	  // Actualizacin de las variables anteriores
	  ekm2 = ekm1;
	  ekm1 = ek;  
	  
	  umi_1km2=umi_1km1;
	  umi_1km1=umi_1k;
	  umi_3km2=umi_3km1;
	  umi_3km1=umi_3k;
	  umi_5km2=umi_5km1;
	  umi_5km1=umi_5k;
	  umi_7km2=umi_7km1;
	  umi_7km1=umi_7k;
	  umi_9km2=umi_9km1;
	  umi_9km1=umi_9k;
	  umi_11km2=umi_11km1;
	  umi_11km1=umi_11k;
	  
	  /*
	  if(uk >= 0.99){
	    uk = 0.99;
	    }
	  if(uk <= -0.99){
	    uk = -0.99;
	    }
	  */
	  
	  //Para lazo abierto
      //uk = rd; 

      uc=uk*(Ncont/4)*(Vbase/Vcc);  //(Vbase/Vcc) Agregar este trmino para implementacin en p.u.
      } //termina el calculo de la estrategia de control


     //**********************************************************//
	 // Incrementamos o decrementamos el contador segun corresponda
     contador = contador + direccion;
   if (contador == 0)
      {
	  direccion=1;
      muestreo=1;
      muestra=1;
      //calculo_u=1;
	  }
   if (contador == Ncont/2)
	  {
	  direccion=-1;
      //muestreo=1;
      //calculo_u=1;
	  }

   if (contador == Ncont/2)  //((cont==0) || (cont==Ncont/2))
      {
      ComparReg1=  uc + Ncont/4;
      ComparReg2= -uc + Ncont/4;
      }

   // Maquina de estado que define el patron PWM
    if (ComparReg1 > contador)
        PWM1=1;
    else
        PWM1=0;

    if (ComparReg2 > contador)
        PWM2=1;
    else
        PWM2=0;

    if(muestreo==1)
    	{
       // Se realiza el muestreo de las variables controladas
        vo = in[4]/Vbase;   // in[4]/Vbase Agregar este trmino para implementacin en p.u.
        iL = in[5]/Ibase;   // in[5]/Ibase Agregar este trmino para implementacin en p.u.
        //vo = -vo;	        // Comentar estas dos lneas siguientes, si se invierten las seales
        //iL = -iL;	        // de disparo que van a las llaves superior e inferior de cada brazo, o quitar el signo menos.
        calculo_u=1;
      }

// Se actualizan los valores de las salidas
	out[0] = PWM1; //brazo 1
	out[1] = PWM2; //brazo 2
	out[2] = uk;
	out[3] = rd;
	out[4] = umi_1k;
	out[5] = ek;
	out[6] = muestra;
	out[7] = contador;
	out[8] = calculo_u;
	
	muestreo=0;
}


BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved)
{
	switch(fdwReason)
	{
		case DLL_PROCESS_ATTACH:
		{
			break;
		}
		case DLL_PROCESS_DETACH:
		{
			break;
		}
		case DLL_THREAD_ATTACH:
		{
			break;
		}
		case DLL_THREAD_DETACH:
		{
			break;
		}
	}
	
	/* Return TRUE on success, FALSE on failure */
	return TRUE;
}
